searchData={"items":[{"type":"task","title":"mix oas.gen.plug","doc":"Generates router and plug handlers for a proper OpenAPI Specification in YAML or JSON file.\n\n    mix oas.gen.plug --file path/to/openapi.yaml\n    mix oas.gen.plug --file path/to/openapi.yml\n    mix oas.gen.plug --file path/to/openapi.json\n\nThe arguments of `oas.gen.plug` mix task:\n\n* `--file`, required, the completed path to the specification file in YAML or JSON format.\n* `--router`, optional, the generated router's module alias, by default it is `Router` (the full module name is `Oasis.Gen.Router` by default), for example we set `--router Hello.MyRouter` meanwhile there is no other special name space defined, the final router module is `Oasis.Gen.Hello.MyRouter` in `/lib/oasis/gen/hello/my_router.ex` path.\n* `--name-space`, optional, the generated all modules' name space, by default it is `Oasis.Gen`, this argument will always override the name space from the input `--file` if any `\"x-oasis-name-space\"` field(s) defined.\n* `--force`, optional, forces creation without a shell prompt.\n* `--quiet`, optional, does not log command output.","ref":"Mix.Tasks.Oas.Gen.Plug.html"},{"type":"function","title":"Mix.Tasks.Oas.Gen.Plug.run/1","doc":"","ref":"Mix.Tasks.Oas.Gen.Plug.html#run/1"},{"type":"module","title":"Oasis.Controller","doc":"Base on `Plug.Builder`, this module can be `use`-d into a module in order to build a plug pipeline:\n\n    defmodule MyApp.HelloController do\n      use Oasis.Controller\n\n      plug(Plug.Parsers,\n        parsers: [:urlencoded],\n        pass: [\"*/*\"]\n      )\n\n      def call(conn, opts) do\n        conn = super(conn, opts)\n        json(conn, %{\"body_params\" => conn.body_params})\n      end\n    end\n\nAnd provide some common functionality for easily use, this realization comes from\n[Phoenix.Controller](https://hexdocs.pm/phoenix/Phoenix.Controller.html).","ref":"Oasis.Controller.html"},{"type":"function","title":"Oasis.Controller.html/2","doc":"Sends html response.","ref":"Oasis.Controller.html#html/2"},{"type":"function","title":"Examples - Oasis.Controller.html/2","doc":"iex> html(conn, \" ... \")","ref":"Oasis.Controller.html#html/2-examples"},{"type":"function","title":"Oasis.Controller.json/2","doc":"Sends JSON response.\n\nIt uses `Jason` to encode the input as an iodata data.","ref":"Oasis.Controller.html#json/2"},{"type":"function","title":"Examples - Oasis.Controller.json/2","doc":"iex> json(conn, %{id: 1})","ref":"Oasis.Controller.html#json/2-examples"},{"type":"function","title":"Oasis.Controller.router_module/1","doc":"Returns the router module as an atom, raises if unavailable.","ref":"Oasis.Controller.html#router_module/1"},{"type":"function","title":"Oasis.Controller.text/2","doc":"Sends text response.","ref":"Oasis.Controller.html#text/2"},{"type":"function","title":"Examples - Oasis.Controller.text/2","doc":"iex> text(conn, \"hello\")\n\n    iex> text(conn, :implements_to_string)","ref":"Oasis.Controller.html#text/2-examples"},{"type":"behaviour","title":"Oasis.HMACToken","doc":"","ref":"Oasis.HMACToken.html"},{"type":"behaviour","title":"Callback - Oasis.HMACToken","doc":"There are two callback functions reserved for use in the generated modules when we use the hmac security\nscheme of the OpenAPI Specification.\n\n* `c:crypto_config/3`, provides a way to define the crypto-related key information for the high level usage,\n  it required to return a `Oasis.HMACToken.Crypto` struct (or nil).\n* `c:verify/3`, an optional function to provide a way to custom the verification of the token, you may\n  want to validate request datetime, HTTP body or other more rules to verify it.","ref":"Oasis.HMACToken.html#module-callback"},{"type":"behaviour","title":"Example - Oasis.HMACToken","doc":"Here is an example to verify that HTTP request time does not exceed the current time by 1 minute.\n\n```elixir\ndefmodule Oasis.Gen.HMACAuth do\n  @behaviour Oasis.HMACToken\n  alias Oasis.HMACToken.Crypto\n\n  # in seconds\n  @max_diff 60\n\n  @impl true\n  def crypto_config(_conn, _opts, _credential) do\n    %Crypto{\n      credential: \"...\",\n      secret: \"...\"\n    }\n  end\n\n  @impl true\n  def verify(conn, token, _opts) do\n    with {:ok, _} <- Oasis.HMACToken.verify(conn, token, opts),\n         {:ok, timestamp} <- conn |> get_header_date() |> parse_header_date() do\n      timestamp_now = DateTime.utc_now() |> DateTime.to_unix()\n\n      if abs(timestamp_now - timestamp) < @max_diff do\n        {:ok, timestamp}\n      else\n        {:error, :expired}\n      end\n    end\n  end\n\n  defp get_header_date(conn) do\n    conn\n    |> Plug.Conn.get_req_header(\"x-oasis-date\")\n    |> case do\n      [date] -> date\n      _ -> nil\n    end\n  end\n\n  defp parse_header_date(str) when is_binary(str) do\n    with {:ok, datetime} <- Timex.parse(str, \"%a, %d %b %Y %H:%M:%S GMT\", :strftime),\n         timestamp when is_integer(timestamp) <- Timex.to_unix(datetime) do\n      {:ok, timestamp}\n    end\n  end\n\n  defp parse_header_date(_otherwise), do: {:error, :expired}\nend\n```","ref":"Oasis.HMACToken.html#module-example"},{"type":"callback","title":"Oasis.HMACToken.crypto_config/3","doc":"","ref":"Oasis.HMACToken.html#c:crypto_config/3"},{"type":"type","title":"Oasis.HMACToken.opts/0","doc":"","ref":"Oasis.HMACToken.html#t:opts/0"},{"type":"function","title":"Oasis.HMACToken.sign/4","doc":"Sign HTTP requests according to settings.","ref":"Oasis.HMACToken.html#sign/4"},{"type":"type","title":"Oasis.HMACToken.token/0","doc":"","ref":"Oasis.HMACToken.html#t:token/0"},{"type":"function","title":"Oasis.HMACToken.verify/3","doc":"Default implementation of the callback `verify`, only verify the signature.","ref":"Oasis.HMACToken.html#verify/3"},{"type":"callback","title":"Oasis.HMACToken.verify/3","doc":"","ref":"Oasis.HMACToken.html#c:verify/3"},{"type":"type","title":"Oasis.HMACToken.verify_error/0","doc":"","ref":"Oasis.HMACToken.html#t:verify_error/0"},{"type":"module","title":"Oasis.HMACToken.Crypto","doc":"In general, when we define a hmac security scheme of the OpenAPI Specification,\nthe generated module will use this struct to define the required crypto-related\nkey information.","ref":"Oasis.HMACToken.Crypto.html"},{"type":"type","title":"Oasis.HMACToken.Crypto.t/0","doc":"","ref":"Oasis.HMACToken.Crypto.html#t:t/0"},{"type":"module","title":"Oasis.Router","doc":"Base on `Plug.Router` to add a pre-parser to convert and validate the path param(s) in the\nfinal generated HTTP verb match functions.\n\nThe generated router module uses `Oasis.Router` to instead of `Plug.Router`, in fact, they don't make a\nhuge difference, except that specify parameters which will then be available and types coverted\nin the function body:\n\n    get(\"/hello/:id\",\n      private: %{\n        path_schema: %{\n          \"id\" => %{\n            \"schema\" => %ExJsonSchema.Schema.Root{\n              schema: %{\"type\" => \"integer\"}\n            }\n          }\n        }\n      }) do\n      # Notice that the `:id` variable is an integer\n      send_resp(conn, 200, \"hello #{id}\")\n    end\n\nThe `:id` parameter will also be available and coverted as an integer in the function body as\n`conn.params[\"id\"]` and `conn.path_params[\"id\"]`.","ref":"Oasis.Router.html"},{"type":"behaviour","title":"Oasis.Token","doc":"A simple wrapper of `Plug.Crypto` to provide a way to generate and verify bearer token for use\nin the [bearer security scheme](https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.1.0.md#securitySchemeObject)\nof the OpenAPI Specification.\n\nWhen we use `sign/2` and `verify/2`, the data stored in the token is signed to\nprevent tampering but not encrypted, in this scenario, we can store identification information\n(such as user NON-PII data), but *SHOULD NOT* be used to store confidential information\n(such as credit card numbers, PIN code).\n\n* \"NON-PII\" means Non Personally Identifiable Information.\n\nIf you don't want clients to be able to determine the value of the token, you may use `encrypt/2`\nand `decrypt/2` to generate and verify the token.","ref":"Oasis.Token.html"},{"type":"behaviour","title":"Callback - Oasis.Token","doc":"There are two callback functions reserved for use in the generated modules when we use the bearer security\nscheme of the OpenAPI Specification.\n\n* `c:crypto_config/2`, provides a way to define the crypto-related key information for the high level usage,\n  it required to return an `Oasis.Token.Crypto` struct.\n* `c:verify/3`, an optional function to provide a way to custom the verification of the token, you may\n  want to use encrypt/decrypt to the token, or other more rules to verify it.","ref":"Oasis.Token.html#module-callback"},{"type":"callback","title":"Oasis.Token.crypto_config/2","doc":"Avoid using the application enviroment as the configuration mechanism for this library,\nand make crypto-related key information configurable when use bearer authentication.\n\nThe `Oasis.Plug.BearerAuth` module invokes this callback function to fetch a predefined\n`Oasis.Token.Crypto` struct, and then use it to verify the bearer token of the request.","ref":"Oasis.Token.html#c:crypto_config/2"},{"type":"function","title":"Oasis.Token.decrypt/2","doc":"A wrapper of `Plug.Crypto.decrypt/4` to use `Oasis.Token.Crypto` to decrypt the original data\nfrom the token and verify its integrity, please see `Plug.Crypto.decrypt/4` for details.","ref":"Oasis.Token.html#decrypt/2"},{"type":"function","title":"Oasis.Token.encrypt/2","doc":"A wrapper of `Plug.Crypto.encrypt/4` to use `Oasis.Token.Crypto` to encode, encrypt and\nsign data into a token you can send to clients, please see `Plug.Crypto.encrypt/4` for details.","ref":"Oasis.Token.html#encrypt/2"},{"type":"type","title":"Oasis.Token.opts/0","doc":"","ref":"Oasis.Token.html#t:opts/0"},{"type":"function","title":"Oasis.Token.random_string/1","doc":"Generates a random string in N length via `:crypto.strong_rand_bytes/1`.","ref":"Oasis.Token.html#random_string/1"},{"type":"function","title":"Oasis.Token.sign/2","doc":"A wrapper of `Plug.Crypto.sign/4` to use `Oasis.Token.Crypto` to sign data\ninto a token you can send to clients, please see `Plug.Crypto.sign/4` for details.","ref":"Oasis.Token.html#sign/2"},{"type":"function","title":"Oasis.Token.verify/2","doc":"A wrapper of `Plug.Crypto.verify/4` to use `Oasis.Token.Crypto` to decode the original\ndata from the token and verify its integrity, please see `Plug.Crypto.verify/4` for details.","ref":"Oasis.Token.html#verify/2"},{"type":"callback","title":"Oasis.Token.verify/3","doc":"An optional callback function to decode the original data from the token, and verify\nits integrity.\n\nIf we use `sign/2` to create a token, sign it, then provide it to a client application,\nthe client will then use this token to authenticate requests for resources from the server,\nin this scenario, as a common use case, the `Oasis.Plug.BearerAuth` module uses `verify/2`\nto finish the verification of the bearer token, so we do not need to implement this\ncallback function in general.\n\nBut if we use `encrypt/2` or other encryption methods to encode, encrypt, and sign data into a token\nand send to clients, we need to implement this callback function to custom the way to decrypt\nthe token and verify its integrity.","ref":"Oasis.Token.html#c:verify/3"},{"type":"type","title":"Oasis.Token.verify_error/0","doc":"","ref":"Oasis.Token.html#t:verify_error/0"},{"type":"module","title":"Oasis.Token.Crypto","doc":"A module to represent crypto-related key information.\n\nAll fields of `Oasis.Token.Crypto` are completely map to:\n\n* `Plug.Crypto.encrypt/4` and `Plug.Crypto.decrypt/4`\n* `Plug.Crypto.sign/4` and `Plug.Crypto.verify/4`\n\nPlease refer the above functions for details to construct it.\n\nIn general, when we define a bearer security scheme of the OpenAPI Specification,\nthe generated module will use this struct to define the required crypto-related\nkey information.\n\nPlease note that the value of the `:secret_key_base` field is required to be a string at least 20 length.","ref":"Oasis.Token.Crypto.html"},{"type":"type","title":"Oasis.Token.Crypto.t/0","doc":"","ref":"Oasis.Token.Crypto.html#t:t/0"},{"type":"module","title":"Oasis.Plug.BearerAuth","doc":"Functionality for providing Bearer HTTP authentication.\n\nIt is recommended to only use this module in production if SSL is enabled and enforced.","ref":"Oasis.Plug.BearerAuth.html"},{"type":"module","title":"Example - Oasis.Plug.BearerAuth","doc":"As any other Plug, we can use the `bearer_auth/2` plug:\n\n    # lib/pre_handler.ex\n    import Oasis.Plug.BearerAuth\n\n    plug :bearer_auth,\n      security: Oasis.Gen.BearerAuth\n      key_to_assigns: :user_id\n\n    # lib/oasis/gen/bearer_auth.ex\n    defmodule Oasis.Gen.BearerAuth do\n      @behaviour Oasis.Token\n\n      @impl true\n      def crypto_config(_conn, _options) do\n        # return a `Oasis.Token.Crypto` struct in your preferred way\n        %Oasis.Token.Crypto{\n          secret_key_base: \"...\",\n          salt: \"...\",\n          max_age: 7200\n        }\n      end\n    end\n\nOr directly plug `Oasis.Plug.BearerAuth`:\n\n    # lib/pre_handler.ex\n\n    plug(\n      Oasis.Plug.BearerAuth,\n      security: Oasis.Gen.BearerAuth,\n      key_to_assigns: :user_id\n    )\n\n    # lib/oasis/gen/bearer_auth.ex\n    defmodule Oasis.Gen.BearerAuth do\n      @behaviour Oasis.Token\n\n      @impl true\n      def crypto_config(_conn, _options) do\n        # ...\n      end\n    end\n\nWe define the [bearer security scheme](https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.1.0.md#securitySchemeObject)\nof the OpenAPI specification in our API design document, and then generate the related module and provide the requried configuration,\nlet's take some examples from YAML specifications.\n\nHere we apply a global security to all operation objects:\n\n    openapi: 3.1.0\n\n    components:\n      securitySchemes:\n        bearerAuth: # arbitrary name for the security scheme\n          type: http\n          scheme: bearer\n          bearerFormat: JWT\n\n    security:\n      - bearerAuth: []\n\nHere we apply a security to a operation object, and define an optional specification extension `\"x-oasis-key-to-assigns\"` field\nto the `:key_to_assigns` option of `bearer_auth/2`:\n\n    openapi: 3.1.0\n\n    components:\n      securitySchemes:\n        bearerAuth: # arbitrary name for the security scheme\n          type: http\n          scheme: bearer\n          bearerFormat: JWT\n          x-oasis-key-to-assigns: user_id\n\n    paths:\n      /something:\n        get:\n          security:\n            - bearerAuth: []\n\nThe above arbitrary name `\"bearerAuth\"` for the security scheme will be transferred into a generated module (see the above mentioned \"Oasis.Gen.BearerAuth\"\nmodule) to provide the required crypto-related configuration, and use it as the value to the `:security` option of `bearer_auth/2`.\n\nBy default, the generated \"BearerAuth\" module will inherit the module name space in order from the paths object, and then the operation object if they defined\nthe `\"x-oasis-name-space\"` field, or defaults to `Oasis.Gen` if there are no any specification defined. As an option, we can add an `\"x-oasis-name-space\"`\nfield as a specification extension of the security scheme object to override the module name space, meanwhile, the optional `--name-space` argument to the\n`mix oas.gen.plug` command line is in the highest priority to set the name space of the generated modules.\n\n    components:\n      securitySchemes:\n        bearerAuth: # arbitrary name for the security scheme\n          type: http\n          scheme: bearer\n          bearerFormat: JWT\n          x-oasis-key-to-assigns: user_id\n          x-oasis-name-space: MyAuth\n\nIn the above example, the final generated module name of `\"bearerAuth\"` is `MyAuth.BearerAuth` when there is no `--name-space` argument of mix task input.\n\nAfter we define the bearer authentication into the specification, then run `mix oas.gen.plug` task with this spec file (via `--file` argument), there will\ngenerate the above similar code to the related module file as long as it does not exist, it also follows the name space definition\nof that module, and the generation does not override it once the file existed, we need to further edit this file to provide a crypto-related\nconfiguration in your preferred way.\n\nIf we need a customization to verify the bearer token, we can implement a callback function `c:Oasis.Token.verify/3` to this scenario.\n\n    # lib/bearer_auth.ex\n    defmodule BearerAuth do\n      @behaviour Oasis.Token\n\n      @impl true\n      def crypto_config(conn, options) do\n        %Oasis.Token.Crypto{\n          ...\n        }\n      end\n\n      @impl true\n      def verify(conn, token, options) do\n        # write your rules to verify the token,\n        # and return the expected results in:\n        #   {:ok, data}, verified\n        #   {:error, :expired}, expired token\n        #   {:error, :invalid}, invalid token\n      end\n    end","ref":"Oasis.Plug.BearerAuth.html#module-example"},{"type":"function","title":"Oasis.Plug.BearerAuth.bearer_auth/2","doc":"High-level usage of Baerer HTTP authentication.\n\nSee the module docs for examples.","ref":"Oasis.Plug.BearerAuth.html#bearer_auth/2"},{"type":"function","title":"Options - Oasis.Plug.BearerAuth.bearer_auth/2","doc":"* `:security`, required, a module be with `Oasis.Token` behaviour.\n  * `:key_to_assigns`, optional, after the verification of the token, the original data\n    will be stored into the `conn.assigns` once this option defined, for example, if set\n    it as `:user_id`, we can access the verified data via `conn.assigns.user_id` in the\n    next plug pipeline.","ref":"Oasis.Plug.BearerAuth.html#bearer_auth/2-options"},{"type":"function","title":"Oasis.Plug.BearerAuth.parse_bearer_auth/1","doc":"Parses the request token from Bearer HTTP authentication.\n\nIt returns either `{:ok, token}` or `{:error, \"invalid_request\"}`.","ref":"Oasis.Plug.BearerAuth.html#parse_bearer_auth/1"},{"type":"function","title":"Oasis.Plug.BearerAuth.request_bearer_auth/2","doc":"Requests bearer authentication from the client.\n\nIt sets the response to status 401 with \"Unauthorized\" as body.\nThe response is not sent though (nor the connection is halted),\nallowing developers to further customize it.\n\nA response example:\n\n    HTTP/1.1 401 Unauthorized\n    www-authenticate: Bearer realm=\"Application\",\n                      error=\"invalid_token\",\n                      error_description=\"the access token expired\"","ref":"Oasis.Plug.BearerAuth.html#request_bearer_auth/2"},{"type":"function","title":"Options - Oasis.Plug.BearerAuth.request_bearer_auth/2","doc":"* `:realm` - the authentication realm. The value is not fully\n    sanitized, so do not accept user input as the realm and use\n    strings with only alphanumeric characters and space.\n  * `:error` - an optional tuple to represent `\"error\"` and `\"error_description\"`\n    attributes, for example, `{\"invalid_token\", \"the access token expired\"}`","ref":"Oasis.Plug.BearerAuth.html#request_bearer_auth/2-options"},{"type":"module","title":"Oasis.Plug.HMACAuth","doc":"Functionality for providing an HMAC HTTP authentication.\n\n*STATEMENT*: Currently there is no standard HMAC authentication definition in the OpenAPI v3.1.0 specification,\nwe implement this function to add an `hmac- ` as the corresponding `scheme` field to `http`\ntype of the security scheme, thanks for some public HMAC services or API design as references:\n\n  * [Azure REST API Authentication HMAC](https://docs.microsoft.com/en-us/azure/azure-app-configuration/rest-api-authentication-hmac)\n  * [AWS S3 sigv4 Authentication](https://docs.aws.amazon.com/AmazonS3/latest/API/sigv4-auth-using-authorization-header.html)\n  * [AWS general sigv4 Authentication example](https://docs.aws.amazon.com/general/latest/gr/sigv4-signed-request-examples.html)\n\nIt is recommended to only use this module in production if SSL is enabled and enforced.","ref":"Oasis.Plug.HMACAuth.html"},{"type":"module","title":"Usage - Oasis.Plug.HMACAuth","doc":"As any other Plug, we can use the `hmac_auth/2` plug:\n\n```elixir\n# lib/pre_handler.ex\nimport Oasis.Plug.HMACAuth\n\nplug(\n  :hmac_auth,\n  signed_headers: \"x-oasis-date;host\",\n  algorithm: :sha256,\n  security: Oasis.Gen.HMACAuth\n)\n\n# lib/oasis/gen/hmac_auth.ex\ndefmodule Oasis.Gen.HMACAuth do\n  @behaviour Oasis.HMACToken\n\n  @impl true\n  def crypto_config(_conn, _options, _credential) do\n    # return `Oasis.HMACToken.Crypto` struct in your preferred way\n    %Oasis.HMACToken.Crypto{\n      credential: \"client id\",\n      secret: \"hmac secret\",\n    }\n  end\nend\n```\n\nOr directly plug `Oasis.Plug.HMACAuth`:\n\n```elixir\n# lib/pre_handler.ex\nplug(\n  Oasis.Plug.HMACAuth,\n  signed_headers: \"x-oasis-date;host\",\n  algorithm: :sha256,\n  security: Oasis.Gen.HMACAuth\n)\n\n# lib/oasis/gen/HMAC_auth.ex\ndefmodule Oasis.Gen.HMACAuth do\n  @behaviour Oasis.HMACToken\n\n  @impl true\n  def crypto_config(_conn, _options, _credential) do\n    ...\n  end\nend\n```\n\nLet's take an example from a YAML specification, we apply a global security to all operation objects:\n\n```yaml\nopenapi: 3.1.0\n\ncomponents:\n  securitySchemes:\n    HMACAuth: # arbitrary name for the security scheme\n      type: http\n      scheme: hmac-sha256\n      x-oasis-signed-headers: x-oasis-date;host\n      x-oasis-name-space: MyAuth\n\nsecurity:\n  - HMACAuth: []\n```\n\nThe above arbitrary name `\"HMACAuth\"` for the security scheme will be transferred into a generated module (see the above mentioned \"Oasis.Gen.HMACAuth\"\nmodule) to provide the required crypto-related configuration, and use it as the value to the `:security` option of `hmac_auth/2`.\n\nBy default, the generated `\"HMACAuth\"` module will inherit the module name space in order from the paths object, and then the operation object if they defined\nan `x-oasis-name-space` field, or defaults to `Oasis.Gen` if there are no any specification defined. As an option, we can add an `x-oasis-name-space`\nfield as a specification extension of the security scheme object to override the generated module's name space, meanwhile, the optional `--name-space` argument to the\n`mix oas.gen.plug` command line is in the highest priority to set the name space of the all generated modules.\n\n```yaml\ncomponents:\n  securitySchemes:\n    HMACAuth: # arbitrary name for the security scheme\n      type: http\n      scheme: hmac-sha256\n      x-oasis-signed-headers: x-oasis-date;host\n      x-oasis-name-space: MyAuth\n```\n\nIn the above example, the final generated module name of `\"HMACAuth\"` is `\"MyAuth.HMACAuth\"` when there is no `--name-space` argument of mix task input.\n\nAfter we define the HMAC authentication into the specification, then run `mix oas.gen.plug` task with this spec file (via `--file` argument), there will\ngenerate the above similar code to the related module file as long as it does not exist, it also follows the name space definition\nto that module, and the generation does not override it once the file existed, we need to further edit this file to provide a crypto-related\nconfiguration in your preferred way.\n\nIf we need a customization to verify the HMAC token, we can implement a callback function `c:Oasis.HMACToken.verify/3` to this scenario.\n\n    # lib/hmac_auth.ex\n    defmodule HMACAuth do\n      @behaviour Oasis.HMACToken\n\n      @impl true\n      def crypto_config(conn, options, credential) do\n        ...\n      end\n\n      @impl true\n      def verify(conn, token, options) do\n        # write your rules to verify the token,\n        # and return the expected results in:\n        #   {:ok, token}, verified\n        #   {:error, :expired}, expired token\n        #   {:error, :invalid_token}, invalid token\n      end\n    end","ref":"Oasis.Plug.HMACAuth.html#module-usage"},{"type":"module","title":"Supported Algorithms - Oasis.Plug.HMACAuth","doc":"Here we use `:crypto.mac/4` to compute a MAC of type `:hmac` from data, the completed supported algorithms can be found in\nthe [hash algorithm type](https://www.erlang.org/doc/man/crypto.html#type-hash_algorithm) of `:crypto`, see\n[Erlang/OTP crypto user's guide - HMAC](https://erlang.org/doc/apps/crypto/algorithm_details.html#hmac) for more details.","ref":"Oasis.Plug.HMACAuth.html#module-supported-algorithms"},{"type":"type","title":"Oasis.Plug.HMACAuth.algorithm/0","doc":"","ref":"Oasis.Plug.HMACAuth.html#t:algorithm/0"},{"type":"function","title":"Oasis.Plug.HMACAuth.hmac_auth/2","doc":"High-level usage of HMAC HTTP authentication.\n\nSee the module docs for examples.","ref":"Oasis.Plug.HMACAuth.html#hmac_auth/2"},{"type":"function","title":"Options - Oasis.Plug.HMACAuth.hmac_auth/2","doc":"* `:algorithm`, required, see [Algorithms Supported](#module-algorithms-supported) for details.\n  * `:security`, required, a module be with `Oasis.HMACToken` behaviour.\n  * `:signed_headers`, required, defines HTTP request headers added to the signature, the provided headers are required in the request,\n    and both in client/server side will use them into signature in the explicit definition order. (e.g., `\"x-oasis-date;host\"`)","ref":"Oasis.Plug.HMACAuth.html#hmac_auth/2-options"},{"type":"type","title":"Oasis.Plug.HMACAuth.opts/0","doc":"","ref":"Oasis.Plug.HMACAuth.html#t:opts/0"},{"type":"function","title":"Oasis.Plug.HMACAuth.parse_hmac_auth/2","doc":"Parses the request token from HMAC HTTP authentication.","ref":"Oasis.Plug.HMACAuth.html#parse_hmac_auth/2"},{"type":"module","title":"Oasis.Plug.RequestValidator","doc":"A plug to convert types and validate the HTTP request parameters by the schemas of\nthe OpenAPI definition.\n\nThe schema options can be found in the generated `pre-` plug handler file, the full list:\n\n  * `:query_schema`\n  * `:header_schema`\n  * `:cookie_schema`\n  * `:body_schema`\n\nAll of these options are fully map and generated from the corresponding definition of the OpenAPI Specification.\n\nWhen the query parameters are verified by the validation of `:query_schema`, the coverted types of query parameters\nare reserved in `:query_params` and `:params` field of the `Plug.Conn`.\n\nWhen the header parameters are verified by the validation of `:header_schema`, the converted types of header parameters\nare reserved in `:req_headers` field of the `Plug.Conn`.\n\nWhen the cookie parameters are verified by the validation of `:cookie_schema`, the coverted types of cookie parameters\nare reserved in `:req_cookies` field of the `Plug.Conn`.\n\nWhen the request body is verified by the validation of `:body_schema`, the coverted types of request body are reserved\nin `:body_params` and `:params` field of the `Plug.Conn`.","ref":"Oasis.Plug.RequestValidator.html"},{"type":"function","title":"Oasis.Plug.RequestValidator.call/2","doc":"","ref":"Oasis.Plug.RequestValidator.html#call/2"},{"type":"function","title":"Oasis.Plug.RequestValidator.init/1","doc":"","ref":"Oasis.Plug.RequestValidator.html#init/1"},{"type":"module","title":"Oasis.BadRequestError.Invalid","doc":"This error is used to indicate could not parse a parameter into the type due to client error.","ref":"Oasis.BadRequestError.Invalid.html"},{"type":"module","title":"Oasis.BadRequestError.InvalidToken","doc":"This error is used to indicate the provided token is expired, revoked, malformed, or invalid.","ref":"Oasis.BadRequestError.InvalidToken.html"},{"type":"module","title":"Oasis.BadRequestError.JsonSchemaValidationFailed","doc":"This error is used to indicate could not pass the validation of the defined json schema.\n\nThis module is an equivalent replacement to `ExJsonSchema.Validator.Error`, we could see more detailed information\nin the `:error` field be with `\"ExJsonSchema.Validator.Error.*\"` modules.","ref":"Oasis.BadRequestError.JsonSchemaValidationFailed.html"},{"type":"module","title":"Oasis.BadRequestError.Required","doc":"This error is used to indicate there missing a required parameter due to client error.","ref":"Oasis.BadRequestError.Required.html"},{"type":"exception","title":"Oasis.BadRequestError","doc":"Error raised when some reason could not process the request due to client error.","ref":"Oasis.BadRequestError.html"},{"type":"exception","title":"Oasis.FileNotFoundError","doc":"Error raised when use an invalid file path to generate corresponding modules.","ref":"Oasis.FileNotFoundError.html"},{"type":"exception","title":"Oasis.InvalidSpecError","doc":"Error raised when use some invalid OpenAPI specification to generate corresponding modules.","ref":"Oasis.InvalidSpecError.html"},{"type":"extras","title":"Oasis","doc":"# Oasis\n\n[![Coverage Status](https://coveralls.io/repos/github/elixir-oasis/oasis/badge.svg?branch=main)](https://coveralls.io/github/elixir-oasis/oasis?branch=main)","ref":"readme.html"},{"type":"extras","title":"Introduction - Oasis","doc":"Background\n\n> The [OpenAPI Specification](https://www.openapis.org/) (OAS) defines a standard, programming language-agnostic interface description for REST APIs, which allows both humans and computers to discover and understand the capabilities of a service without requiring access to source code, additional documentation, or inspection of network traffic. When properly defined via OpenAPI, a consumer can understand and interact with the remote service with a minimal amount of implementation logic. Similar to what interface descriptions have done for lower-level programming, the OpenAPI Specification removes guesswork in calling a service.\n\nOasis is:\n\nBase on `Plug`'s implements, according to the OpenAPI Specification to generate server's router and the all well defined HTTP request handlers, since the OAS defines a detailed collection of data types by [JSON Schema Specification](https://json-schema.org/), Oasis leverages this and focuses on the types conversion and validation to the parameters of the HTTP request.\n\n* Maintain an OpenAPI Specification document(in YAML or JSON) is the first priority\n* Generate the maintainable router and HTTP request handlers code by the defined document\n* In general, we do not need to manually write the OpenAPI definitions in Elixir\n* Simplify the REST APIs to convert types and validate the parameters of the HTTP request\n* More reference and communication to your REST APIs","ref":"readme.html#introduction"},{"type":"extras","title":"Installation - Oasis","doc":"Add `oasis` as a dependency to your mix.exs\n\n```elixir\ndef deps do\n  [\n    {:oasis, \"~> 0.3\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Implements to OAS - Oasis","doc":"Oasis does not cover the full OpenAPI specification, so far the implements contain:\n\n* [Components Object](https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.1.0.md#componentsObject)\n* [Paths Object](https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.1.0.md#pathsObject)\n* [Path Item Object](https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.1.0.md#pathItemObject)\n* [Operation Object](https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.1.0.md#operationObject)\n* [Parameter Object](https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.1.0.md#parameterObject)\n* [Request Body Object](https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.1.0.md#requestBodyObject)\n* [Media Type Object](https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.1.0.md#mediaTypeObject)\n* [Header Object](https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.1.0.md#headerObject)\n* [Reference Object](https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.1.0.md#referenceObject)\n* [Schema Object](https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.1.0.md#schemaObject)\n* [Security Scheme Object](https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.1.0.md#securitySchemeObject)\n  * Bearer Authentication, please see `Oasis.Plug.BearerAuth` for details.\n  * HMAC Authentication, please see `Oasis.Plug.HMACAuth` for details.\n\nWe also have some OpenAPI Specification Extensions defined for use, please see [our Specification Extensions Guide](https://hexdocs.pm/oasis/specification_ext.html).","ref":"readme.html#implements-to-oas"},{"type":"extras","title":"How to use - Oasis","doc":"","ref":"readme.html#how-to-use"},{"type":"extras","title":"Prepare YAML or JSON specification - Oasis","doc":"First, write your API document refer the OpenAPI Specification(see [reference](#reference) for details), we build Oasis with the version [3.1.0](http://spec.openapis.org/oas/v3.1.0) of the OAS, as usual, the common use case should be covered for the version `3.0.*` of the OAS.\n\nHere is a minimum specification for an example in YAML, we save it as \"petstore-mini.yaml\" in this tutorial.\n\n```yaml\nopenapi: \"3.1.0\"\ninfo:\n  title: Petstore Mini\npaths:\n  /pets:\n    get:\n      parameters:\n        - name: tags\n          in: query\n          required: false\n          schema:\n            type: array\n            items:\n              type: string\n        - name: limit\n          in: query\n          requried: false\n          schema:\n            type: integer\n            format: int32\n      response:\n        '200':\n          content:\n            application/json:\n              schema:\n                type: array\n                items:\n                  $ref: '#/components/schemas/Pet'\n    post:\n      operationId: addPet\n      requestBody:\n        required: true\n        content:\n          application/json:\n            schema:\n              $ref: '#/components/schemas/NewPet'\n      response:\n        '200':\n          content:\n            application/json:\n              schema:\n                $ref: '#/components/schemas/Pet'\n\n  /pets/{id}:\n    get:\n      operationId: find pet by id\n      parameters:\n        - name: id\n          in: path\n          required: true\n          schema:\n            type: integer\n            format: int64\n      response:\n        '200':\n          content:\n            application/json:\n              schema:\n                $ref: '#/components/schemas/Pet'\ncomponents:\n  schemas:\n    Pet:\n      allOf:\n        - $ref: '#/components/schemas/NewPet'\n        - type: object\n          required:\n          - id\n          properties:\n            id:\n              type: integer\n              format: int64\n\n    NewPet:\n      type: object\n      required:\n        - name\n      properties:\n        name:\n          type: string\n        tag:\n          type: string\n```","ref":"readme.html#prepare-yaml-or-json-specification"},{"type":"extras","title":"Run mix task - Oasis","doc":"And then we run the following mix task to generate the corresponding files:\n\n```\nmix oas.gen.plug --file path/to/petstore-mini.yaml\n```\n\nWe will see these output:\n\n```\nGenerates Router and Plug modules from OAS\n* creating lib/oasis/gen/router.ex\n* creating lib/oasis/gen/pre_find_pet_by_id.ex\n* creating lib/oasis/gen/find_pet_by_id.ex\n* creating lib/oasis/gen/pre_add_pet.ex\n* creating lib/oasis/gen/add_pet.ex\n* creating lib/oasis/gen/pre_get_pets.ex\n* creating lib/oasis/gen/get_pets.ex\n```\n\nThe arguments of `oas.gen.plug` mix task:\n\n* `--file`, required, the completed path to the specification file in YAML or JSON format.\n* `--router`, optional, the generated router's module alias, by default it is `Router` (the full module name is `Oasis.Gen.Router` by default), for example we set `--router Hello.MyRouter` meanwhile there is no other special name space defined, the final router module is `Oasis.Gen.Hello.MyRouter` in `/lib/oasis/gen/hello/my_router.ex` path.\n* `--name-space`, optional, the generated all modules' name space, by default it is `Oasis.Gen`, this argument will always override the name space from the input `--file` if any `\"x-oasis-name-space\"` field(s) defined.","ref":"readme.html#run-mix-task"},{"type":"extras","title":"Special instructions - Oasis","doc":"#### Name Plug's handler\n\nRefer the OAS, the `operationId` field of [Operation Object](https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.1.0.md#operationObject) is not required, but it should be unique among all operations described in the API if `operationId` field exists.\n\n   * When use this field, Oasis will use it to construct the generated module alias and the file name of \".ex\" file, e.g. see the above `find_pet_by_id.ex` file with `Oasis.Gen.FindPetById` module name.\n   * When not use this filed, Oasis will combine the HTTP verb with the URL to generate the module alias and the file name of \".ex\" file, e.g. see the above `get_pets.ex` file with `Oasis.Gen.GetPets` module name.\n\n#### Generated code to folder\n\nThe generated codes are always put in the `lib` directory of your application root path, because they need to be integrated into your application in runtime.\n\n#### Custom name space of generated modules\n\nBy default, the name space of the generated module is `Oasis.Gen` and its folder path is `lib/oasis/gen`, but we can custom it use these ways:\n\n1. As a final global definition when run `oas.gen.plug` mix task, input `--name-space` argument to set this, this argument always overrides all name space even we have some sections of document have special definition (via `\"x-oasis-name-space\"`), for example:\n\n  ```\n  mix oas.gen.plug --file path/to/petstore-mini.yaml --name-space My.Petstore\n  Generates Router and Plug modules from OAS\n  * creating lib/my/petstore/router.ex\n  * creating lib/my/petstore/pre_find_pet_by_id.ex\n  * creating lib/my/petstore/find_pet_by_id.ex\n  * creating lib/my/petstore/pre_add_pet.ex\n  * creating lib/my/petstore/add_pet.ex\n  * creating lib/my/petstore/pre_get_pets.ex\n  * creating lib/my/petstore/get_pets.ex\n  ```\n\n  Now, we can see the generation folder path is `lib/my/petstore`, let's open the `get_pets.ex` file, the module name changed from `Oasis.Gen.GetPets` to `My.Petstore.GetPets`.\n\n2. Use `\"x-oasis-name-space\"` in the OAS's [Operation Object](https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.1.0.md#operationObject), for example, add this newline `x-oasis-name-space: Common.Api` as below:\n\n  ```yaml\n    paths:\n      /pets:\n        get:\n          x-oasis-name-space: Common.Api\n          parameters:\n            - name: tags\n              ...\n        post:\n          ...\n      /pets/{id}:\n        ...\n  ```\n\n  Run again, please note this time no `--name-space` argument.\n\n  ```\n  mix oas.gen.plug --file path/to/petstore-mini.yaml\n  Generates Router and Plug modules from OAS\n  * creating lib/oasis/gen/router.ex\n  * creating lib/oasis/gen/pre_find_pet_by_id.ex\n  * creating lib/oasis/gen/find_pet_by_id.ex\n  * creating lib/oasis/gen/pre_add_pet.ex\n  * creating lib/oasis/gen/add_pet.ex\n  * creating lib/common/api/pre_get_pets.ex\n  * creating lib/common/api/get_pets.ex\n  ```\n\n  We can see `pre_get_pets.ex` and `get_pets.ex` files are moved into the expected path, and their module names are `Common.Api.GetPets` and `Common.Api.PreGetPets`, other operations do not define any `\"x-oasis-name-space\"` field, so they still use the default one `Oasis.Gen`.\n\n3. Use `\"x-oasis-name-space\"` in the OAS's [Paths Object](https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.1.0.md#pathsObject), this use case as a global setting and can archive it in the document under the file version management, for example, add this newline `x-oasis-name-space: Common.Api` as below:\n\n  ```yaml\n    paths:\n      x-oasis-name-space: Common.Api\n      /pets:\n        get:\n          parameters:\n            - name: tags\n              ...\n        post:\n          ...\n      /pets/{id}:\n        ...\n  ```\n\n  Run again, please note this time no `--name-space` argument, and the GET operation of the \"/pets\" path is no \"x-oasis-name-space\" either.\n\n  ```\n  mix oas.gen.plug --file path/to/petstore-mini.yaml\n  Generates Router and Plug modules from OAS\n  * creating lib/common/api/router.ex\n  * creating lib/common/api/pre_find_pet_by_id.ex\n  * creating lib/common/api/find_pet_by_id.ex\n  * creating lib/common/api/pre_add_pet.ex\n  * creating lib/common/api/add_pet.ex\n  * creating lib/common/api/pre_get_pets.ex\n  * creating lib/common/api/get_pets.ex\n  ```\n\nWe can see all generated files are moved into the expected path, and all modules' name start with `Common.Api`.\n\nSummarize about the name space of generated module:\n\n  1. The optional `--name-space` argument to the `mix oas.gen.plug` command line is in the highest priority to set the name space;\n\n  2. We also can define `\"x-oasis-name-space\"` extension field in the specification document to make this archivable, please see [here](https://hexdocs.pm/oasis/specification_ext.html#module-name-space) for details.\n\n#### HTTP request handler files in pairs\n\nThe generated HTTP request handler files are named in pairs, one is `pre_operation.ex`, and another is `operation.ex`, the name beginning with **`pre_`** file is in charge of converting and validating the parameters of each HTTP request definition, the content of this file will be updated according to the section of the document changes or Oasis upgrade in the future, so we ***CAN NOT*** write any business logic in this file; another file is only created in the first time of generation as long as this file does not exist, this is a common `Plug` module which is after the defined preprocessor by `Plug`'s pipeline, we need to fill in business logic in here.\n\n```\nmix oas.gen.plug --file path/to/petstore-mini.yaml\nGenerates Router and Plug modules from OAS\n* creating lib/oasis/gen/router.ex\n* creating lib/oasis/gen/pre_find_pet_by_id.ex\n* creating lib/oasis/gen/find_pet_by_id.ex\n* creating lib/oasis/gen/pre_add_pet.ex\n* creating lib/oasis/gen/add_pet.ex\n* creating lib/oasis/gen/pre_get_pets.ex\n* creating lib/oasis/gen/get_pets.ex\n```\n\n#### Integration\n\nAfter the above generation, let's integrate it, assume the generated router is `Oasis.Gen.Router`, we can use it in the `Plug`'s adapter like this:\n\n```elixir\nPlug.Adapters.Cowboy.child_spec(\n  scheme: :http,\n  plug: Oasis.Gen.Router,\n  options: [\n    port: port\n  ]\n)\n```\n\nOr we can plug this router into the existence router module like this:\n\n```elixir\ndefmodule MyExistenceRouter do\n  use Plug.Router\n\n  plug(Oasis.Gen.Router)\n\n  plug(:match)\n  plug(:dispatch)\n\n  # other\n\nend\n```\n\nPlease note that the added line of `plug(Oasis.Gen.Router)` is before the line of `plug(:match)`.","ref":"readme.html#special-instructions"},{"type":"extras","title":"TODO - Oasis","doc":"1. The Specification contains XML object is not supported so far.\n2. Make document more clear.\n3. There are still some details maybe not implement(or bug) from the OAS, please create an issue or a PR for tracking, thanks in advanced :)","ref":"readme.html#todo"},{"type":"extras","title":"Reference - Oasis","doc":"1. The official OpenAPI Specification v3: [Github](https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.1.0.md) | [Official Web Site](http://spec.openapis.org/oas/v3.1.0)\n2. [JSON Schema Reference](https://json-schema.org/understanding-json-schema/reference/)","ref":"readme.html#reference"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"v0.5.2 (2025-02-20) - Changelog","doc":"* Fix compile warning in Elixir 1.18\n* Fix the broken source url in `mix.exs`","ref":"changelog.html#v0-5-2-2025-02-20"},{"type":"extras","title":"v0.5.1 (2023-02-14) - Changelog","doc":"* Fix to ensure validation and parse working when request header content type with charset ([issue#15](https://github.com/elixir-oasis/oasis/issues/15))\n* Fix to adapt request body (via `Plug.Parsers.JSON`) validation in a `\"_json\"` key wrapper ([issue#19](https://github.com/elixir-oasis/oasis/issues/19))\n* Adapt to use `conn.host` for HMAC authorization when use Plug `\"1.14.0\"` ([PR#18](https://github.com/elixir-oasis/oasis/pull/18))\n* Update compile compatibility with Elixir `~> 1.14` ([PR#21](https://github.com/elixir-oasis/oasis/pull/21))","ref":"changelog.html#v0-5-1-2023-02-14"},{"type":"extras","title":"v0.5.0 (2022-07-21) - Changelog","doc":"* Fix test failed in OTP24\n* Add HMAC based authentication implement ([issue#8](https://github.com/elixir-oasis/oasis/issues/8))\n* Some fixing and enhancement ([PR#12](https://github.com/elixir-oasis/oasis/pull/12))","ref":"changelog.html#v0-5-0-2022-07-21"},{"type":"extras","title":"v0.4.3 (2021-05-13) - Changelog","doc":"* Fix to make properly handle file uploads","ref":"changelog.html#v0-4-3-2021-05-13"},{"type":"extras","title":"v0.4.2 (2021-05-11) - Changelog","doc":"* Add `--force` and `--quiet` options for mix oas.gen.plug","ref":"changelog.html#v0-4-2-2021-05-11"},{"type":"extras","title":"v0.4.1 (2021-04-29) - Changelog","doc":"Fix unexpected \"...\" string in generated `pre_*` module when a large number of parameters defined","ref":"changelog.html#v0-4-1-2021-04-29"},{"type":"extras","title":"v0.4.0 (2021-04-14) - Changelog","doc":"* Improve errors handle and add a guide about it","ref":"changelog.html#v0-4-0-2021-04-14"},{"type":"extras","title":"v0.3.1 (2021-04-08) - Changelog","doc":"* Fix incorrectly handle errors in generated plug module\n* Simplify `handle_errors/2` process in generated `pre_*` module","ref":"changelog.html#v0-3-1-2021-04-08"},{"type":"extras","title":"v0.3.0 (2021-04-08) - Changelog","doc":"* Add `conn.private.oasis_router`\n* Add a specification extensions guide\n* Support Security Scheme Object with Bearer Authentication\n* Fix the order to override `x-oasis-name-space` field","ref":"changelog.html#v0-3-0-2021-04-08"},{"type":"extras","title":"v0.2.1 (2021-03-24) - Changelog","doc":"* Fix unexpected `:body_schema` in generated `pre_*` module","ref":"changelog.html#v0-2-1-2021-03-24"},{"type":"extras","title":"v0.2.0 (2021-03-23) - Changelog","doc":"* Use `Oasis.Controller`","ref":"changelog.html#v0-2-0-2021-03-23"},{"type":"extras","title":"v0.1.0 (2021-03-17) - Changelog","doc":"* Implement some parts of OpenAPI definition `*Object` in parse\n* Implement a basic router and plugs pipeline process\n* Add a mix task `mix task oas.gen.plug` to generate code\n* 100% test coverage","ref":"changelog.html#v0-1-0-2021-03-17"},{"type":"extras","title":"Handle Errors","doc":"# Handle Errors\n\nWhen we run `mix oas.gen.plug` mix task to generate the corresponding modules, we could see the following similar output (here are only for an example)\n\n```\n...\n* creating lib/oasis/gen/pre_list_pets.ex\n* creating lib/oasis/gen/list_pets.ex\n...\n```\n\nThe `\"pre_list_pets.ex\"` and `\"list_pets.ex\"` files are in a pair, the former starts with `\"pre_\"` is in charge of the parse and validation to the request,\nthe latter is after the former's pipeline process, the latter is in charge of the detailed business logic to write, and provide a `handle_errors/2`\nfunction to process the errors:\n\n```\ndef handle_errors(conn, %{kind: _kind, reason: reason, stack: _stack}) do\n  message = Map.get(reason, :message) || \"Something went wrong\"\n  send_resp(conn, conn.status, message)\nend\n```\n\nThe `reason` is an `Oasis.BadRequestError` exception with these errors:\n\n  * `Oasis.BadRequestError.Invalid`\n  * `Oasis.BadRequestError.Required`\n  * `Oasis.BadRequestError.JsonSchemaValidationFailed`\n  * `Oasis.BadRequestError.InvalidToken`\n\nWe can use pattern math to process the potential errors like this:\n\n```\ndef handle_errors(conn, %{kind: _kind, reason: {\n  error: %Oasis.BadRequestError.JsonSchemaValidationFailed{} = error} = reason, stack: _stack}) do\n  # inspect the `error` and `reason` to find more information about this error case.\n  ...\nend\ndef handle_errors(conn, %{kind: _kind, reason: {\n  error: %Oasis.BadRequestError.InvalidToken{} = error} = reason, stack: _stack}) do\n  ...\nend\ndef handle_errors(conn, %{kind: _kind, reason: reason, stack: _stack}) do\n  ...\nend\n```","ref":"handle_errors.html"},{"type":"extras","title":"Specification Extensions","doc":"# Specification Extensions\n\nOasis be with the following [specification extensions](https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.1.0.md#specificationExtensions) to accommodate some use cases in the defined YAML or JSON specification file.","ref":"specification_ext.html"},{"type":"extras","title":"Module Name Space - Specification Extensions","doc":"`\"x-oasis-name-space\"`, optional, use this field to define the generated Elixir module's name space in the following objects, defaults to `Oasis.Gen`:\n  * [Paths Object](https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.1.0.md#pathsObject)\n  * [Operation Object](https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.1.0.md#operationObject)\n  * [Security Scheme Object](https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.1.0.md#securitySchemeObject)","ref":"specification_ext.html#module-name-space"},{"type":"extras","title":"Example - Specification Extensions","doc":"#### In Paths Object\n\nHere, define this field in [Paths Object](https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.1.0.md#pathsObject)\nas a global name space to the all generated modules meanwhile no other special definitions, in the below example\nthere will use \"Hello.Petstore\" to the module naming prefix alias of the all generated modules.\n\n```YAML\npaths:\n  /pets:\n    get:\n      ...\n  /pets/{id}:\n    get:\n      ...\n    delete:\n      ...\n  x-oasis-name-space: Hello.Petstore\n```\n\n#### In Operation Object\n\nHere, define this field in [Operation Object](https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.1.0.md#operationObject),\nit only affect to a operation related generated modules meanwhile no other special definitions, in the below example\nthere will use \"Petstore.Api\" to the module naming prefix alias of the generated related modules to handler request of \"/pets\" in HTTP GET verb, other generated modules\nwill use `Oasis.Gen` as the default.\n\n```YAML\npaths:\n  /pets:\n    get:\n      x-oasis-name-space: Petstore.Api\n      operationId: list pets\n      ...\n  /pets/{id}:\n    get:\n      ...\n    delete:\n      ...\n```\n\n#### In Security Scheme Object\n\nHere, define this field in [Security Scheme Object](https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.1.0.md#securitySchemeObject),\nit only affect to the generated bearer token module meanwhile no other special definitions, in the below example\nthere will use \"Petstore.MyToken\" to the \"helloBearerAuth\" module (its full module name is \"Petstore.MyToken.HelloBearerAuth\").\n\n```YAML\npaths:\n  /pets:\n    get:\n      security:\n        - helloBearerAuth: []\n\ncomponents:\n  securitySchemes:\n    helloBearerAuth: # arbitrary name for the security scheme\n      scheme: bearer\n      type: http\n      x-oasis-name-space: Petstore.MyToken\n```","ref":"specification_ext.html#example"},{"type":"extras","title":"Summary - Specification Extensions","doc":"`\"x-oasis-name-space\"` can be used at the same time for the above scenarios, but the defined in Security Scheme Object will override the defined in Operation Object for\nthe generated bearer token module, the defined in Operation Object will override the defined in Paths Object for the corresponding operation related modules.\n\nWe also can set a global name space when run `mix oas.gen.plug` with the `--name-space` argument, it applies to all generated modules and always override the defined from\nthe specification file.\n\nFollow Elixir's module naming conventions, each dot(\".\") of module name will split the alias as a directory in lowercase to concat the full directory path,\nand also we do not validate the value of this field in a strict mode, so please follow the usual naming conventions as much as possible.","ref":"specification_ext.html#summary"},{"type":"extras","title":"Router Module Alias - Specification Extensions","doc":"`\"x-oasis-router\"`, optional, use this field to define the generated Elixir router module's alias in [Paths Object](https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.1.0.md#pathsObject), defaults to `Router`.","ref":"specification_ext.html#router-module-alias"},{"type":"extras","title":"Example - Specification Extensions","doc":"When use an OpenAPI Specification to run `mix oas.gen.plug`, there will generate a router file and some corresponding operation handler modules files(in pair),\nthe router file packages all defined HTTP router(s) to each hanlder(s), we can custom the module name of the router file by the `\"x-oasis-router\"` and\n`\"x-oasis-name-space\"` in the specification file, without other special name space defined, the following example defines the module full name space of the router\nmodule is \"Oasis.Gen.HelloRouter\".\n\n```YAML\npaths:\n  /pets:\n    ..\n  /pets/{id}:\n    ..\n  x-oasis-router: HelloRouter\n```","ref":"specification_ext.html#example"},{"type":"extras","title":"Summary - Specification Extensions","doc":"We also can set a router name when run `mix oas.gen.plug` with the `--router` argument, it applies to the router module and always override the defined from the specification file.\n\nFollow Elixir's module naming conventions, each dot(\".\") of module name will split the alias as a directory in lowercase to concat the full directory path,\nand also we do not validate the value of this field in a strict mode, so please follow the usual naming conventions as much as possible.","ref":"specification_ext.html#summary"},{"type":"extras","title":"Save Data from Bearer Token - Specification Extensions","doc":"`\"x-oasis-key-to-assigns\"`, optional, after the verification of the token, the original data will be stored into this provided field (as an atom) of the `conn.assigns` for the next accessing, if not defined it, there won't do any process for the verified original data, please see `Oasis.Plug.BearerAuth` for details.","ref":"specification_ext.html#save-data-from-bearer-token"},{"type":"extras","title":"Example - Specification Extensions","doc":"Once the input token is verified, we can access the original data(decrypted from token) via `conn.assigns.user_id` in the next plug's pipeline.\n\n```YAML\nopenapi: 3.1.0\n\ncomponents:\n  securitySchemes:\n    bearerAuth: # arbitrary name for the security scheme\n      type: http\n      scheme: bearer\n      bearerFormat: JWT\n      x-oasis-key-to-assigns: user_id\n\npaths:\n  /something:\n    get:\n      security:\n        - bearerAuth: []\n```","ref":"specification_ext.html#example"},{"type":"extras","title":"Signed Headers for HAMC Authentication - Specification Extensions","doc":"`\"x-oasis-signed-headers\"`, requried when use HMAC authentication in `security` scheme object, this field represents which HTTP header(s) will be used\ninto signature, please see [HMAC-based authentication](hmac_based_authentication.html) document for details.","ref":"specification_ext.html#signed-headers-for-hamc-authentication"},{"type":"extras","title":"HMAC-based Authentication","doc":"# HMAC-based Authentication","ref":"hmac_based_authentication.html"},{"type":"extras","title":"Backgroupd - HMAC-based Authentication","doc":"*STATEMENT*: Currently there is no standard HMAC authentication definition in the OpenAPI v3.1.0 specification,\nwe implement this function to add an `hmac- ` as the corresponding `scheme` field to `http`\ntype of the security scheme object in YAML/JSON specification, thanks for some public HMAC services or API design as references:\n\n  * [Azure REST API Authentication HMAC](https://docs.microsoft.com/en-us/azure/azure-app-configuration/rest-api-authentication-hmac)\n  * [AWS S3 sigv4 Authentication](https://docs.aws.amazon.com/AmazonS3/latest/API/sigv4-auth-using-authorization-header.html)\n  * [AWS general sigv4 Authentication example](https://docs.aws.amazon.com/general/latest/gr/sigv4-signed-request-examples.html)\n\nWe need a mechanism based on the [HTTP Authorization Header](https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Authorization) to\nauthenticate access permissions to meet the following requirements:\n\n1. Make sure that the request is from a trusted client.\n2. Make sure that the some key HTTP headers are not tampered.\n3. *(Optional)* Make sure that the HTTP body is not tampered.\n4. *(Optional)* Make sure that the interval between the request send-time and the server received-time does not exceed a certain critical value.\n\nOasis provides a complete set of scaffolding to easily build such a set of certification services.","ref":"hmac_based_authentication.html#backgroupd"},{"type":"extras","title":"Protocol - HMAC-based Authentication","doc":"Recommend HTTP request sent be transmitted over `TLS/SSL` on production.","ref":"hmac_based_authentication.html#protocol"},{"type":"extras","title":"HTTP Authorization Header - HMAC-based Authentication","doc":"The client request must contains an `Authorization` HTTP header in this syntax:\n\n    Authorization: HMAC-  Credential= &SignedHeaders= &Signature= \n\n* ` `, the algorithm name(in uppercase, e.g. `SHA256`) used to compute a Message Authentication Code of type hmac, supported algorithms please\n  see [here](Oasis.Plug.HMACAuth.html#module-supported-algorithms). \n* `Credential`, the ID of the access key used to identify for server.\n* `SignedHeaders`, the HTTP header names separated by semicolons(`;`), the values of these headers are required to signature the request in your defined order,\n  the values of these HTTP headers should be properly provided in this request as well, do not use white spaces for the HTTP header name.\n* `Signature`, a hash   with the `String-to-Sign`, and the value(aka `Secret`) of the access key to the `Credential`, and then base64 encode the hashed result.\n\nAbout the `String-to-Sign`, please see as below:\n\n*String-to-Sign* =\n  *HTTP_METHOD* + \"\\n\" +\n  *path_and_query_string* + \"\\n\" +\n  *values_of_signed_headers*\n\n| Variable | Description |\n| --- | --- |\n| *HTTP_METHOD* | The HTTP method of the request in uppercase, e.g. \"POST\" |\n| *path_and_query_string* | The absolute URI with query string(if existed) of the request, e.g. \"/foo/bar?a=1\" |\n| *values_of_signed_headers* | The values of all HTTP request headers listed in `SignedHeaders`, use semicolons(`;`) to separate the values in order of `SignedHeaders`. |\n\nExample:\n\n    String-to-Sign =\n        \"POST\" + \"\\n\" +\n        \"/new?version=1\" + \"\\n\" +\n        \"2021-11-24 06:43:20.393420Z;foo.bar.host;{\\\"name\\\":\\\"test\\\",\\\"type\\\":1}\"\n\nLet's take this pair of access key for example:\n\nAccess Key ID | Access Key Secret\n----- | -----\n*mykey_abc* | *123456789*\n\nNotice: the value of the access key secret should be only the client and server know it, and DO NOT use in plaintext in any transmission.\n\nThe corresponding HTTP headers of the request are:\n\n    Host: foo.bar.host\n    Date: 2021-11-24 06:43:20.393420Z\n    Authorization: HMAC-SHA256 Credential=mykey_abc&SignedHeaders=date;host;body&Signature=oSBomxpJWcwlhVkif5LV80zecDLpts9Z13+cth1NKV4=\n    Body: \"{\\\"name\\\":\\\"test\\\",\\\"type\\\":1}\"\n\nRefer the best practices, please try to keep the dynamic part of request into the signature.\n\nThe way to concatenate a `String-To-Sign` is fixed, but there is a flexible customization to define your HTTP headers to sign via `SignedHeaders` field,\nproperly keep the value(s) of the signed header(s) for the server side's verification, here are some pseudocode for example:\n\n    x-oasis-content-sha256: base64_encode(SHA256(body))\n    x-oasis-content-md5: base64_encode(MD5(body))","ref":"hmac_based_authentication.html#http-authorization-header"},{"type":"extras","title":"Tutorial - HMAC-based Authentication","doc":"","ref":"hmac_based_authentication.html#tutorial"},{"type":"extras","title":"1. Define schema in OpenAPI Specification - HMAC-based Authentication","doc":"```yaml\n# priv/oas/main.yaml\n\nopenapi: \"3.1.0\"\ninfo:\n  title: My API Title\n  version: \"1.0\"\npaths:\n  /test_hmac:\n    post:\n      # this API (/test_hmac) should be authenticated by HMAC-based Authentication\n      security:\n        - HMACAuth: [ ]\n      requestBody:\n        content:\n          application/json:\n            schema:\n              type: object\n              properties:\n                a:\n                  type: string\n      responses:\n        \"200\":\n          description: OK\n  /other:\n    get:\n      # and this API will not be authenticated \n      responses:\n        \"200\":\n          description: OK\n    \ncomponents:\n  securitySchemes:\n    HMACAuth:\n      type: http\n      # will use hmac hash sha256 algorithm\n      scheme: hmac-sha256\n      # will verify host, request datetime and body hash\n      x-oasis-signed-headers: host;x-oasis-date;x-oasis-body-sha256\n```\n\nThe `x-oasis-signed-headers` field is a specification extension to define which HTTP header(s) will be used into signature.\n\nSome HTTP client libraries don't let you set the HTTP \"Date\" header, in this case, we can define an arbitrary naming(should be started with \"x-\") to represent\ndate related, for example, the above specification example defines an `x-oasis-date` HTTP header.\n\nIn the above specification example, the `x-oasis-body-sha256` is a customization HTTP header field, you can name it in a arbitrary naming(should be started with \"x-\").","ref":"hmac_based_authentication.html#1-define-schema-in-openapi-specification"},{"type":"extras","title":"2. Generate Code - HMAC-based Authentication","doc":"```shell\n# in the shell\nmix oas.gen.plug --file priv/oas/main.yaml\n```\n\nThen you will get some similar generated files:\n\n```elixir\n# lib/oasis/gen/pre_post_test_hmac.ex\ndefmodule Oasis.Gen.PrePostTestHMAC do\n  # NOTICE: Please DO NOT write any business code in this module, since it will always be overridden when\n  # run `mix oas.gen.plug` task command with the OpenAPI Specification file.\n  use Oasis.Controller\n  use Plug.ErrorHandler\n\n  plug(\n    Plug.Parsers,\n    parsers: [:json],\n    json_decoder: Jason,\n    pass: [\"*/*\"],\n    body_reader: {Oasis.CacheRawBodyReader, :read_body, []}\n  )\n\n  plug(\n    Oasis.Plug.RequestValidator,\n    body_schema: %{\n      \"content\" => %{\n        \"application/json\" => %{\n          \"schema\" => %ExJsonSchema.Schema.Root{\n            schema: %{\"properties\" => %{\"a\" => %{\"type\" => \"string\"}}, \"type\" => \"object\"}\n          }\n        }\n      }\n    }\n  )\n\n  plug(\n    Oasis.Plug.HMACAuth,\n    signed_headers: \"host;x-oasis-date;x-oasis-body-sha256\",\n    algorithm: :sha256,\n    security: Oasis.Gen.HMACAuth\n  )\n\n  def call(conn, opts) do\n    conn |> super(opts) |> Oasis.Gen.PostTestHMACWithBody.call(opts) |> halt()\n  end\n\n  defdelegate handle_errors(conn, error), to: Oasis.Gen.PostTestHMACWithBody\nend\n```\n\n```elixir\n# lib/oasis/gen/hmac_auth.ex\ndefmodule Oasis.Gen.HMACAuth do\n  # NOTICE: This module is generated when run `mix oas.gen.plug` task command with the OpenAPI Specification file\n  # in the first time, and then it WILL NOT be modified in the future generation command(s) once this file exists,\n  # please write the crypto-related configuration to the bearer token in this module.\n  @behaviour Oasis.HMACToken\n  alias Oasis.HMACToken.Crypto\n\n  @impl true\n  def crypto_config(_conn, _opts, _credential) do\n    # ...\n    nil\n  end\n\n  @impl true\n  def verify(conn, token, opts) do\n    with {:ok, _} <- Oasis.HMACToken.verify(conn, token, opts) do\n      {:ok, token}\n    end\n  end\nend\n```","ref":"hmac_based_authentication.html#2-generate-code"},{"type":"extras","title":"3. Implement your `crypto_config/3` and `verify/3` callbacks. - HMAC-based Authentication","doc":"```elixir\n# lib/oasis/gen/hmac_auth.ex\ndefmodule Oasis.Gen.HMACAuth do\n  # NOTICE: This module is generated when run `mix oas.gen.plug` task command with the OpenAPI Specification file\n  # in the first time, and then it WILL NOT be modified in the future generation command(s) once this file exists,\n  # please write the crypto-related configuration to the bearer token in this module.\n  @behaviour Oasis.HMACToken\n  alias Oasis.HMACToken.Crypto\n  \n  # in seconds\n  @max_diff 60\n\n  @impl true\n  def crypto_config(_conn, _opts, _credential) do\n    # Here just an example\n    # You should provide these sensitive information from config file or database\n    %Crypto{\n      credential: \"mykey_abc\",\n      secret: \"123456789\"\n    }\n  end\n\n  @impl true\n  def verify(conn, token, opts) do\n    with {:ok, _} <- Oasis.HMACToken.verify(conn, token, opts),\n         {:ok, _} <- verify_date(conn, token, opts),\n         {:ok, _} <- verify_body(conn, token, opts) do\n      {:ok, token}\n  end\n  \n  defp verify_date(conn, token, _opts) do\n    with {:ok, timestamp} <- conn |> get_header(\"x-oasis-date\") |> parse_header_date() do\n      timestamp_now = DateTime.utc_now() |> DateTime.to_unix()\n\n      if abs(timestamp_now - timestamp) < @max_diff do\n        {:ok, timestamp}\n      else\n        {:error, :expired}\n      end\n    end\n  end\n  \n  defp verify_body(conn, token, opts) do\n    algorithm = opts[:algorithm]\n    raw_body = conn.assigns.raw_body\n    crypto = crypto_config(conn, opts, token.credential)\n    body_hmac = hmac(algorithm, crypto.secret, raw_body)\n    body_hmac_header = get_header(conn, \"x-oasis-body-sha256\")\n\n    if body_hmac == body_hmac_header do\n      {:ok, token}\n    else\n      {:error, :invalid_token}\n    end\n  end\n\n  defp get_header(conn, key) do\n    conn\n    |> Plug.Conn.get_req_header(key)\n    |> case do\n      [value] -> value\n      _ -> nil\n    end\n  end\n  \n  defp parse_header_date(str) when is_binary(str) do\n    with {:ok, datetime} <- Timex.parse(str, \"%a, %d %b %Y %H:%M:%S GMT\", :strftime),\n         timestamp when is_integer(timestamp) <- Timex.to_unix(datetime) do\n      {:ok, timestamp}\n    end\n  end\n\n  defp parse_header_date(_otherwise), do: {:error, :expired}\n\n  defp hmac(subtype, secret, content) do\n    Base.encode64(:crypto.mac(:hmac, subtype, secret, content))\n  end\nend\n```","ref":"hmac_based_authentication.html#3-implement-your-crypto_config-3-and-verify-3-callbacks"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.2"}}